@@ -34,12 +34,13 @@ eb_initialize_multi_searches(book)
 
     subbook = book->subbook_current;
 
-    for (i = 0, multi = subbook->multis; i < EB_MAX_KEYWORDS;
+    for (i = 0, multi = subbook->multis; i < EB_MAX_MULTI_SEARCHES;
 	 i++, multi++) {
 	eb_initialize_search(&multi->search);
+	multi->title[0] = '\0';
 	multi->entry_count = 0;
 	for (j = 0, entry = multi->entries;
-	     j < multi->entry_count; j++, entry++) {
+	     j < EB_MAX_MULTI_ENTRIES; j++, entry++) {
 	    eb_initialize_search(entry);
 	}
     }
@@ -93,7 +94,6 @@ eb_load_multi_searches(book)
     char *buffer_p;
     int index_count;
     int index_id;
-    int page;
     int i, j, k;
 
     LOG(("in: eb_load_multi_searches(book=%d)", book->code));
@@ -147,21 +147,29 @@ eb_load_multi_searches(book)
 		 * Get the index page information of the entry.
 		 */
 		index_id = eb_uint1(buffer_p);
-		page = eb_uint4(buffer_p + 2);
 		switch (index_id) {
 		case 0x71:
-		    if (entry->start_page == 0)
-			entry->start_page = page;
-		    entry->index_id = index_id;
-		    break;
 		case 0x91:
 		case 0xa1:
-		    entry->start_page = page;
+		    if (entry->start_page != 0 && entry->index_id != 0x71)
+			break;
+		    entry->start_page = eb_uint4(buffer_p + 2);
+		    entry->end_page = entry->start_page
+			+ eb_uint4(buffer_p + 6) - 1;
 		    entry->index_id = index_id;
+		    entry->katakana         = EB_INDEX_STYLE_ASIS;
+		    entry->lower            = EB_INDEX_STYLE_CONVERT;
+		    entry->mark             = EB_INDEX_STYLE_ASIS;
+		    entry->long_vowel       = EB_INDEX_STYLE_ASIS;
+		    entry->double_consonant = EB_INDEX_STYLE_ASIS;
+		    entry->contracted_sound = EB_INDEX_STYLE_ASIS;
+		    entry->voiced_consonant = EB_INDEX_STYLE_ASIS;
+		    entry->small_vowel      = EB_INDEX_STYLE_ASIS;
+		    entry->p_sound          = EB_INDEX_STYLE_ASIS;
+		    entry->space            = EB_INDEX_STYLE_ASIS;
 		    break;
 		case 0x01:
-		    entry->candidates_page = page;
-		    entry->index_id = index_id;
+		    entry->candidates_page = eb_uint4(buffer_p + 2);
 		    break;
 		}
 		buffer_p += 16;
@@ -181,6 +189,146 @@ eb_load_multi_searches(book)
 }
 
 
+/*
+ * Default multi search titles (written in JIS X 0208).
+ */
+static const char *default_multi_titles_jisx0208[] = {
+    "J#9g8!:w#1",    /* Multi search 1. */
+    "J#9g8!:w#2",    /* Multi search 2. */
+    "J#9g8!:w#3",    /* Multi search 3. */
+    "J#9g8!:w#4",    /* Multi search 4. */
+    "J#9g8!:w#5",    /* Multi search 5. */
+    "J#9g8!:w#6",    /* Multi search 6. */
+    "J#9g8!:w#7",    /* Multi search 7. */
+    "J#9g8!:w#8",    /* Multi search 8. */
+    "J#9g8!:w#9",    /* Multi search 9. */
+    "J#9g8!:w#1#0",  /* Multi search 10. */
+};
+
+/*
+ * Default multi search titles (written in ASCII, subset of ISO 8859-1).
+ */
+static const char *default_multi_titles_latin[] = {
+    "Multi search 1",
+    "Multi search 2",
+    "Multi search 3",
+    "Multi search 4",
+    "Multi search 5",
+    "Multi search 6",
+    "Multi search 7",
+    "Multi search 8",
+    "Multi search 9",
+    "Multi search 10",
+};
+
+/*
+ * Load multi search titles.
+ */
+EB_Error_Code
+eb_load_multi_titles(book)
+    EB_Book *book;
+{
+    EB_Error_Code error_code;
+    EB_Subbook *subbook;
+    char buffer[EB_SIZE_PAGE];
+    int title_count;
+    char *title;
+    size_t offset;
+    int i;
+
+    LOG(("in: eb_load_multi_searches(book=%d)", book->code));
+
+    subbook = book->subbook_current;
+
+    /*
+     * Set default titles.
+     */
+    if (book->character_code == EB_CHARCODE_ISO8859_1) {
+	for (i = 0; i < subbook->multi_count; i++) {
+	    title = subbook->multis[i].title;
+	    strcpy(title, default_multi_titles_latin[i]);
+	}
+    } else {
+	for (i = 0; i < subbook->multi_count; i++) {
+	    title = subbook->multis[i].title;
+	    strcpy(title, default_multi_titles_jisx0208[i]);
+	    eb_jisx0208_to_euc(title, title);
+	}
+    }
+
+    if (book->disc_code != EB_DISC_EPWING || subbook->search_title_page == 0)
+	goto succeeded;
+
+    /*
+     * Read the page of the multi search.
+     */
+    if (zio_lseek(&subbook->text_zio, 
+	(off_t)(subbook->search_title_page - 1) * EB_SIZE_PAGE, SEEK_SET)
+	< 0) {
+	error_code = EB_ERR_FAIL_SEEK_TEXT;
+	goto failed;
+    }
+    if (zio_read(&subbook->text_zio, buffer, EB_SIZE_PAGE) != EB_SIZE_PAGE) {
+	error_code = EB_ERR_FAIL_READ_TEXT;
+	goto failed;
+    }
+
+    title_count = eb_uint2(buffer);
+    if (EB_MAX_SEARCH_TITLES < title_count)
+	title_count = EB_MAX_SEARCH_TITLES;
+
+    /*
+     * We need titles for multi searches only.
+     *     titles[ 0]: title for word and endword searches.
+     *     titles[ 1]: title for keyword search.
+     *     titles[ 2]: common title for all multi searches.
+     *                (we don't need this)
+     *     titles[ 3]: title for multi search 1.
+     *         :
+     *     titles[12]: title for multi search 10.
+     *     titles[13]: title for menu search.
+     *
+     * The offset of titles[3] is:
+     *     the number of entries(2bytes)
+     *     + reserved 1 (68bytes)
+     *     + title for word and endword searches (70bytes)
+     *     + title for keyword search (70bytes)
+     *     + common title for all multi searches (70bytes)
+     *     + reserved 2 (70bytes)
+     *     = 2 + 68 + 70 + 70 + 70 + 70 = 350
+     */
+    for (i = 4, offset = 350; i < EB_MAX_SEARCH_TITLES; i++, offset += 70) {
+	if (subbook->multi_count <= i - 4)
+	    break;
+	if (eb_uint2(buffer + offset) != 0x02)
+	    continue;
+
+	/*
+	 * Each titles[] consists of
+	 *    parameter (2bytes)
+	 *    short title (16bytes)
+	 *    long title (32bytes)
+	 * We get long title rather than short one.
+	 */
+	title = subbook->multis[i - 4].title;
+	strncpy(title, buffer + offset + 2 + 16, EB_MAX_MULTI_TITLE_LENGTH);
+	title[EB_MAX_MULTI_TITLE_LENGTH] = '\0';
+	eb_jisx0208_to_euc(title, title);
+    }
+
+succeeded:
+    LOG(("out: eb_load_multi_titles() = %s", eb_error_string(EB_SUCCESS)));
+    return EB_SUCCESS;
+
+    /*
+     * An error occurs...
+     */
+  failed:
+    LOG(("out: eb_load_multi_titles() = %s", eb_error_string(error_code)));
+    return error_code;
+}
+
+
 /*
  * Examine whether the current subbook in `book' supports `MULTI SEARCH'
  * or not.
@@ -216,6 +364,66 @@ eb_have_multi_search(book)
 }
 
 
+/*
+ * Return a title of the multi search `multi_id'.
+ */
+EB_Error_Code
+eb_multi_title(book, multi_id, title)
+    EB_Book *book;
+    EB_Multi_Search_Code multi_id;
+    char *title;
+{
+    EB_Error_Code error_code;
+    EB_Subbook *subbook;
+
+    eb_lock(&book->lock);
+    LOG(("in: eb_multi_title(book=%d, multi_id=%d)",
+	(int)book->code, (int)multi_id));
+
+    /*
+     * The book must have been bound.
+     */
+    if (book->path == NULL) {
+	error_code = EB_ERR_UNBOUND_BOOK;
+	goto failed;
+    }
+
+    /*
+     * Current subbook must have been set.
+     */
+    subbook = book->subbook_current;
+    if (subbook == NULL) {
+	error_code = EB_ERR_NO_CUR_SUB;
+	goto failed;
+    }
+
+    /*
+     * `multi_id' must be a valid code.
+     */
+    if (multi_id < 0 || subbook->multi_count <= multi_id) {
+	error_code = EB_ERR_NO_SUCH_MULTI_ID;
+	goto failed;
+    }
+
+    strcpy(title, subbook->multis[multi_id].title);
+
+    LOG(("out: eb_multi_title(title=%s) = %s", title,
+	eb_error_string(EB_SUCCESS)));
+    eb_unlock(&book->lock);
+
+    return EB_SUCCESS;
+
+    /*
+     * An error occurs...
+     */
+  failed:
+    *title = '\0';
+    LOG(("out: eb_multi_title() = %s", eb_error_string(error_code)));
+    eb_unlock(&book->lock);
+    return error_code;
+}
+
+
 /*
  * Return a list of multi search ids in `book'.
  */
