@@ -93,7 +93,7 @@ eb_initialize_text_context(book)
     book->text_context.narrow_flag = 0;
     book->text_context.printable_count = 0;
     book->text_context.file_end_flag = 0;
-    book->text_context.text_end_flag = 0;
+    book->text_context.text_status = EB_TEXT_STATUS_CONTINUED;
     book->text_context.skip_code = SKIP_CODE_NONE;
     book->text_context.auto_stop_code = -1;
     book->text_context.candidate[0] = '\0';
@@ -139,7 +139,7 @@ eb_reset_text_context(book)
     book->text_context.narrow_flag = 0;
     book->text_context.printable_count = 0;
     book->text_context.file_end_flag = 0;
-    book->text_context.text_end_flag = 0;
+    book->text_context.text_status = EB_TEXT_STATUS_CONTINUED;
     book->text_context.skip_code = SKIP_CODE_NONE;
     book->text_context.auto_stop_code = -1;
     book->text_context.candidate[0] = '\0';
@@ -297,7 +297,8 @@ eb_read_text(book, appendix, hookset, container, text_max_length, text,
     if (hookset != NULL)
 	eb_lock(&hookset->lock);
     LOG(("in: eb_read_text(book=%d, appendix=%d, text_max_length=%ld)",
-	(int)book->code, (int)appendix->code, (long)text_max_length));
+	(int)book->code, (appendix != NULL) ? (int)appendix->code : 0,
+	(long)text_max_length));
 
     /*
      * Current subbook must have been set and START file must exist.
@@ -401,7 +402,8 @@ eb_read_heading(book, appendix, hookset, container, text_max_length, text,
     if (hookset != NULL)
 	eb_lock(&hookset->lock);
     LOG(("in: eb_read_heading(book=%d, appendix=%d, text_max_length=%ld)",
-	(int)book->code, (int)appendix->code, (long)text_max_length));
+	(int)book->code, (appendix != NULL) ? (int)appendix->code : 0,
+	(long)text_max_length));
 
     /*
      * Current subbook must have been set and START file must exist.
@@ -578,8 +580,8 @@ eb_read_text_internal(book, appendix, hookset, container, text_max_length,
     pthread_mutex_lock(&cache_mutex);
     LOG(("in: eb_read_text_internal(book=%d, appendix=%d, \
 text_max_length=%ld, forward=%d)",
-	(int)book->code, (int)appendix->code, (long)text_max_length,
-	forward_only));
+	(int)book->code, (appendix != NULL) ? (int)appendix->code : 0,
+	(long)text_max_length, forward_only));
 
     /*
      * Initialize variables.
@@ -616,7 +618,7 @@ text_max_length=%ld, forward=%d)",
     /*
      * Return immediately if text-end-flag has been set.
      */
-    if (context->text_end_flag)
+    if (context->text_status != EB_TEXT_STATUS_CONTINUED)
 	goto succeeded;
 
     /*
@@ -703,7 +705,7 @@ text_max_length=%ld, forward=%d)",
 
 	    case 0x03:
 		/* end of text (don't set `in_step') */
-		context->text_end_flag = 1;
+		context->text_status = EB_TEXT_STATUS_HARD_STOP;
 		if (forward_only) {
 		    error_code = EB_ERR_END_OF_CONTENT;
 		    goto failed;
@@ -749,7 +751,7 @@ text_max_length=%ld, forward=%d)",
 		if (0 < context->printable_count
 		    && context->code == EB_TEXT_TEXT) {
 		    if (eb_is_stop_code(book, appendix, argv[0], argv[1])) {
-			context->text_end_flag = 1;
+			context->text_status = EB_TEXT_STATUS_SOFT_STOP;
 			goto succeeded;
 		    }
 		}
@@ -761,7 +763,7 @@ text_max_length=%ld, forward=%d)",
 		/* newline */
 		in_step = 2;
 		if (context->code == EB_TEXT_HEADING) {
-		    context->text_end_flag = 1;
+		    context->text_status = EB_TEXT_STATUS_SOFT_STOP;
 		    context->location += in_step;
 		    goto succeeded;
 		}
@@ -810,7 +812,6 @@ text_max_length=%ld, forward=%d)",
 		break;
 
 	    case 0x1a: case 0x1b: case 0x1c: case 0x1d: case 0x1e: case 0x1f:
-	    case 0xe0:
 		/* emphasis; described in JIS X 4081-1996 */
 		in_step = 4;
 		if (cache_rest_length < in_step) {
@@ -845,8 +846,25 @@ text_max_length=%ld, forward=%d)",
 		hook = hookset->hooks + EB_HOOK_BEGIN_MPEG;
 		break;
 
+	    case 0x3c:
+		/* beginning of inline color graphic */
+		in_step = 20;
+		if (cache_rest_length < in_step) {
+		    error_code = EB_ERR_UNEXP_TEXT;
+		    goto failed;
+		}
+		argc = 4;
+		argv[1] = eb_uint2(cache_p + 2);
+		argv[2] = eb_bcd4(cache_p + 14);
+		argv[3] = eb_bcd2(cache_p + 18);
+		if (argv[1] >> 8 == 0x00)
+		    hook = hookset->hooks + EB_HOOK_BEGIN_IN_COLOR_BMP;
+		else
+		    hook = hookset->hooks + EB_HOOK_BEGIN_IN_COLOR_JPEG;
+		break;
+
 	    case 0x35: case 0x36: case 0x37: case 0x38: case 0x3a:
-	    case 0x3b: case 0x3c: case 0x3d: case 0x3e: case 0x3f:
+	    case 0x3b: case 0x3d: case 0x3e: case 0x3f:
 		in_step = 2;
 		context->skip_code = eb_uint1(cache_p + 1) + 0x20;
 		break;
@@ -864,7 +882,7 @@ text_max_length=%ld, forward=%d)",
 		if (0 < context->printable_count
 		    && context->code == EB_TEXT_TEXT) {
 		    if (eb_is_stop_code(book, appendix, argv[0], argv[1])) {
-			context->text_end_flag = 1;
+			context->text_status = EB_TEXT_STATUS_SOFT_STOP;
 			goto succeeded;
 		    }
 		}
@@ -1016,6 +1034,16 @@ text_max_length=%ld, forward=%d)",
 		hook = hookset->hooks + EB_HOOK_END_MPEG;
 		break;
 
+	    case 0x5c:
+		/* end of inline color graphic */
+		in_step = 2;
+		if (cache_rest_length < in_step) {
+		    error_code = EB_ERR_UNEXP_TEXT;
+		    goto failed;
+		}
+		hook = hookset->hooks + EB_HOOK_END_IN_COLOR_GRAPHIC;
+		break;
+
 	    case 0x61:
 		/* end of keyword */
 		in_step = 2;
@@ -1104,8 +1132,28 @@ text_max_length=%ld, forward=%d)",
 		context->skip_code = eb_uint1(cache_p + 1) + 0x20;
 		break;
 
+	    case 0xe0:
+		/* character modification */
+		in_step = 4;
+		if (cache_rest_length < in_step) {
+		    error_code = EB_ERR_UNEXP_TEXT;
+		    goto failed;
+		}
+		argc = 2;
+		argv[1] = eb_uint2(cache_p + 2);
+		hook = hookset->hooks + EB_HOOK_BEGIN_DECORATION;
+
+		/* Some old EB books don't take an argument. */
+		if (book->disc_code != EB_DISC_EPWING
+		    && eb_uint1(cache_p + 2) >= 0x1f) {
+		    in_step = 2;
+		    hook = &null_hook;
+		}
+		break;
+
 	    case 0xe1:
 		in_step = 2;
+		hook = hookset->hooks + EB_HOOK_END_DECORATION;
 		break;
 
 	    case 0xe4: case 0xe6: case 0xe8: case 0xea: case 0xec: case 0xee:
@@ -1413,9 +1461,12 @@ eb_is_text_stopped(book)
     if (book->subbook_current != NULL) {
 	if (book->text_context.code == EB_TEXT_HEADING
 	    || book->text_context.code == EB_TEXT_TEXT
-	    || book->text_context.code == EB_TEXT_OPTIONAL_TEXT)
-	    is_stopped = (book->text_context.text_end_flag
-		&& book->text_context.unprocessed == NULL);
+	    || book->text_context.code == EB_TEXT_OPTIONAL_TEXT) {
+	    if (book->text_context.text_status != EB_TEXT_STATUS_CONTINUED
+		&& book->text_context.unprocessed == NULL) {
+		is_stopped = 1;
+	    }
+	}
     }
 
     LOG(("out: eb_is_text_stopped() = %d", is_stopped));
@@ -1662,7 +1713,7 @@ eb_forward_text(book, appendix)
 
     eb_lock(&book->lock);
     LOG(("in: eb_forward_text(book=%d, appendix=%d)", (int)book->code,
-	(int)appendix->code));
+	(appendix != NULL) ? (int)appendix->code : 0));
 
     /*
      * Current subbook must have been set and START file must exist.
@@ -1687,13 +1738,12 @@ eb_forward_text(book, appendix)
 	goto failed;
     }
 
-    /*
-     * if the text-end flag has been set, we simply unset the flag
-     * and returns immediately.
-     */
-    if (book->text_context.text_end_flag) {
-	book->text_context.text_end_flag = 0;
+    if (book->text_context.text_status == EB_TEXT_STATUS_SOFT_STOP) {
+	book->text_context.text_status = EB_TEXT_STATUS_CONTINUED;
 	goto succeeded;
+    } else if (book->text_context.text_status == EB_TEXT_STATUS_HARD_STOP) {
+	error_code = EB_ERR_END_OF_CONTENT;
+	goto failed;
     }
 
     /*
@@ -1738,13 +1788,12 @@ eb_forward_heading(book)
     eb_lock(&book->lock);
     LOG(("in: eb_forward_heading(book=%d)", (int)book->code));
 
-    /*
-     * if the text-end flag has been set, we simply unset the flag
-     * and returns immediately.
-     */
-    if (book->text_context.text_end_flag) {
-	book->text_context.text_end_flag = 0;
+    if (book->text_context.text_status == EB_TEXT_STATUS_SOFT_STOP) {
+	book->text_context.text_status = EB_TEXT_STATUS_CONTINUED;
 	goto succeeded;
+    } else if (book->text_context.text_status == EB_TEXT_STATUS_HARD_STOP) {
+	error_code = EB_ERR_END_OF_CONTENT;
+	goto failed;
     }
 
     /*
@@ -1803,7 +1852,7 @@ eb_backward_text(book, appendix)
 
     eb_lock(&book->lock);
     LOG(("in: eb_backward_text(book=%d, appendix=%d)", (int)book->code,
-	(int)appendix->code));
+	(appendix != NULL) ? (int)appendix->code : 0));
 
     /*
      * Current subbook must have been set and START file must exist.
@@ -1832,7 +1881,7 @@ eb_backward_text(book, appendix)
      * Forward text to get auto-stop-code and location where the current
      * text stops.
      */
-    if (book->text_context.text_end_flag) {
+    if (book->text_context.text_status != EB_TEXT_STATUS_CONTINUED) {
 	forward_location = book->text_context.location;
     } else {
 	memcpy(&saved_context, &book->text_context, sizeof(EB_Text_Context));
