@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1998  Motoyuki Kasahara
+ * Copyright (c) 1998, 2000, 01  
+ *    Motoyuki Kasahara
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +24,7 @@
 #endif
 
 #include <sys/types.h>
+#include <sys/stat.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -34,10 +36,10 @@
 mode_t
 get_umask()
 {
-    mode_t um;
+    mode_t current_umask;
 
-    um = umask(022);
-    umask(um);
+    current_umask = umask((mode_t)022);
+    umask(current_umask);
 
-    return um;
+    return current_umask;
 }
